#include <stdio.h>
#include <machine.h>
#include <math.h> 
#include <comedi.h>
#include <comedilib.h>

void comedi_digitalin(
  integer *flag, 
  integer *nevprt,
  double *t,
  double xd[],
  double x[],
  integer *nx,
  double z[],
  integer *nz,
  double tvec[],
  integer *ntvec,
  double rpar[],
  integer *nrpar,
  integer ipar[],
  integer *nipar,
  double *inptr[],
  integer insz[],
  integer *nin,
  double *outptr[],
  integer outsz[],
  integer *nout)

  /*
    ipar[0..n]  = allocated channels
    z[i*7+0] = comedi fd
    z[i*7+1] = comedi subdev
    z[i*7+2] = comedi chan
    z[i*7+3] = comedi maxdata
    z[i*7+4] = comedi min
    z[i*7+5] = comedi max
    z[i*7+6] = comedi delta
  */
{
  // printf("comedi_digitalin: %d %d %f\n", *flag, *nevprt, *t);
  switch (*flag) {
    case 1: { /* Update outputs */
      int i;

      for (i = 0 ; i < outsz[0] ; i++) {
	lsampl_t data;
	comedi_t *cf = (comedi_t*)((int)z[i*7+0]);
        
	comedi_data_read(cf, z[i*7+1], z[i*7+2], 0, 0, &data);
	outptr[0][i] = z[i*7+4] + z[i*7+6] * data; /* min + delta * data */
      }
    } break;
    case 4: { /* Initialization */
      /* Sanity check */
      if (*nz != *nipar * 7 || outsz[0] != *nipar) {
	char error[200];
	
	*flag = -1;
	snprintf(error, sizeof(error),
		 "'comedi_digitalin' parameter/state mismatch\r\n"
		 "  nz      = %d (%d expected)\r\n"
		 "  outsz[0] = %d (%d expected)\r\n",
		 *nz, *nipar * 7, outsz[0], *nipar);
	sciprint(error);
      } else {
	FILE *conf;
	int i;
	
	for (i = 0 ; i < outsz[0] ; i++) { z[i*7+0] = 0; /* channel closed */}
	conf = fopen("/etc/stork.conf", "r");
	if (conf == 0) {
	  sciprint("'comedi_digitalin' could not open '/etc/stork.conf'\r\n");
	  *flag = -1;
	} else {
	  for (i = 0 ; i < outsz[0] ; i++) {
	    char *data, buffer[80];
	    int wanted_channel = ipar[i];
    
	    fseek(conf, 0, SEEK_SET);
	    while (data = fgets(buffer, sizeof(buffer), conf)) {
	      char name[80];
	      int channel, subdev, subchan, err;

	      if (sscanf(data, "din %d %s %d:%d", 
			 &channel, &name, &subdev, &subchan) == 4 &&
		  channel == wanted_channel) {
		comedi_t *cf;
		int flags, range;
		
		cf = comedi_open(name);
		if (!cf) {
		  z[i*7+0] = 0;
		  z[i*7+1] = 0;
		  z[i*7+2] = 0;
		  z[i*7+3] = 0;
		  z[i*7+4] = 0;
		  z[i*7+5] = 0;
		  z[i*7+6] = 0;
		} else {
		  lsampl_t maxdata;
		  comedi_range *range;
		  
		  maxdata = comedi_get_maxdata(cf, subdev, subchan);
		  z[i*7+0] = (int)cf;
		  z[i*7+1] = subdev;
		  z[i*7+2] = subchan;
		  z[i*7+3] = maxdata;
		  z[i*7+4] = 0;
		  z[i*7+5] = 1;
		  z[i*7+6] = (z[i*7+5]-z[i*7+4]) / maxdata;
		}
		break;
	      }
	    }
	    if ((int)z[i*7+0] == 0) {
	      char error[80];
	      snprintf(error, sizeof(error),
		       "Failed to open output channel %d\r\n", 
		       ipar[i]); 
	      sciprint(error);
	      *flag = -1;
	    }
	  }
	  fclose(conf);
	}
      }
    } break;
    case 5: { /* Ending */
      int i;

      for (i = 0 ; i < outsz[0] ; i++) {
	comedi_t *cf = (comedi_t*)((int)z[i*7+0]);
	if (cf) {
	  int data;
	  data = (0 - z[i*7+4]) / z[i*7+6];
	  if (data < 0) { 
	    data = 0; 
	  } else if (data > z[i*7+3]) {
	    data = z[i*7+3];
	  }
	  comedi_data_write(cf, (int)z[i*7+1], (int)z[i*7+2], 0, 0, data);    
	  comedi_close(cf);
	}
      }
    } break;
  }
  return;
}

